# @Coding: UTF-8
# @Time: 2024/9/24 12:58
# @Author: xieyang_ls
# @Filename: annotation.py

from argparse import ArgumentTypeError

from http.server import BaseHTTPRequestHandler

from pyutils_spirit.exception.exception import NoneSignatureError

from pyutils_spirit.spirit_container.spirit_application_container import SpiritApplicationContainer


class ContainerAnnotation:
    spirit_application_container: SpiritApplicationContainer = SpiritApplicationContainer()

    @classmethod
    def resource(cls, names: list[str] | str) -> callable:
        if not isinstance(names, list | str):
            raise ValueError("Resource: the signature of type must be list or str")
        if not all(isinstance(name, str) for name in names):
            raise ValueError("All elements in the list must be strings")
        if len(names) == 0:
            raise ValueError("the names must not be empty")

        def decorator_func(func):
            def wrapper(*args, **kwargs):
                if isinstance(names, str):
                    resources = cls.spirit_application_container.get_resource(signature=names)
                    if resources is None:
                        raise ValueError(f"the signature {names} does not exist")
                else:
                    resources = kwargs["resources"]
                    for name in names:
                        instance = cls.spirit_application_container.get_resource(signature=name)
                        if instance is None:
                            raise ValueError(f"the signature {name} does not exist")
                        resources[name] = instance
                func(args[0], resources)

            wrapper.__decorator__ = "Resource"
            return wrapper

        return decorator_func

    @classmethod
    def get(cls, path: str):
        if not isinstance(path, str):
            raise ValueError('GET Method: path should be a string')
        if len(path) == 0:
            raise ValueError('GET Method: path should not be empty')

        def decorator_get_func(func):
            func.__decorator__ = "GET"
            func.__decorator_path__ = path
            return func

        return decorator_get_func

    @classmethod
    def post(cls, path: str):
        if not isinstance(path, str):
            raise ValueError('POST Method: path should be a string')
        if len(path) == 0:
            raise ValueError('POST Method: path should not be empty')

        def decorator_post_func(func):
            func.__decorator__ = "POST"
            func.__decorator_path__ = path
            return func

        return decorator_post_func

    @classmethod
    def put(cls, path: str):
        if not isinstance(path, str):
            raise ValueError('PUT Method: path should be a string')
        if len(path) == 0:
            raise ValueError('PUT Method: path should not be empty')

        def decorator_put_func(func):
            func.__decorator__ = "PUT"
            func.__decorator_path__ = path
            return func

        return decorator_put_func

    @classmethod
    def delete(cls, path: str):
        if not isinstance(path, str):
            raise ValueError('DELETE Method: path should be a string')
        if len(path) == 0:
            raise ValueError('DELETE Method: path should not be empty')

        def decorator_delete_func(func):
            func.__decorator__ = "DELETE"
            func.__decorator_path__ = path
            return func

        return decorator_delete_func


class Component:
    def __init__(self, signature: str) -> None:
        if not isinstance(signature, str):
            raise NoneSignatureError
        if len(signature) == 0:
            raise ValueError("Component: Signature cannot be empty")
        self.__signature: str = signature

    def __call__(self, component_cls: type[object]) -> type[object]:
        component_cls.__decorator__ = "Component"

        def wrapper_component(__cls: type[object]) -> object:
            __component: object = (ContainerAnnotation.spirit_application_container
                                   .get_resource(signature=self.__signature))
            if __component is None:
                __component = object.__new__(__cls)
                ContainerAnnotation.spirit_application_container.set_resource(signature=self.__signature,
                                                                              resource=__component)
            return __component

        component_cls.__new__ = wrapper_component
        return component_cls


class Mapper:
    def __init__(self, signature: str) -> None:
        if not isinstance(signature, str):
            raise NoneSignatureError
        if len(signature) == 0:
            raise ValueError("Mapper: Signature cannot be empty")
        self.__signature: str = signature

    def __call__(self, mapper_cls: type[object]) -> type[object]:
        mapper_cls.__decorator__ = "Mapper"

        def wrapper_mapper(__cls: type[object]) -> object:
            __mapper: object = (ContainerAnnotation.spirit_application_container
                                .get_resource(signature=self.__signature))
            if __mapper is None:
                __mapper = object.__new__(__cls)
                ContainerAnnotation.spirit_application_container.set_resource(signature=self.__signature,
                                                                              resource=__mapper)
            return __mapper

        mapper_cls.__new__ = wrapper_mapper
        return mapper_cls


class Service:
    def __init__(self, signature: str) -> None:
        if not isinstance(signature, str):
            raise NoneSignatureError
        if len(signature) == 0:
            raise ValueError("Service: Signature cannot be empty")
        self.__signature: str = signature

    def __call__(self, service_cls: type[object]) -> type[object]:
        service_cls.__decorator__ = "Service"

        def wrapper_service(__cls: type[object]) -> object:
            __service: object = (ContainerAnnotation.spirit_application_container
                                 .get_resource(signature=self.__signature))
            if __service is None:
                __service = object.__new__(__cls)
                ContainerAnnotation.spirit_application_container.set_resource(signature=self.__signature,
                                                                              resource=__service)
            return __service

        service_cls.__new__ = wrapper_service
        return service_cls


class Controller:
    def __init__(self, path: str) -> None:
        if not isinstance(path, str):
            raise NoneSignatureError
        if len(path) == 0:
            raise ValueError("Controller: path cannot be empty")
        self.__path: str = path

    def __call__(self, controller_cls: type[object]) -> type[object]:
        controller_cls.__decorator__ = "Controller"
        controller_cls.__decorator_path__ = self.__path

        def wrapper_controller(__cls: type[object]) -> object:
            __controller: object = (ContainerAnnotation.spirit_application_container
                                    .get_resource(signature=self.__path))
            if __controller is None:
                __controller = object.__new__(__cls)
                ContainerAnnotation.spirit_application_container.set_resource(signature=self.__path,
                                                                              resource=__controller)
            return __controller

        controller_cls.__new__ = wrapper_controller
        return controller_cls


class ExceptionAdvice:
    def __init__(self, exception_advice_cls: type) -> None:
        if not isinstance(exception_advice_cls, type):
            raise TypeError('ExceptionAdvice can only be applied to classes')
        exception_advice_cls.__decorator__ = "ExceptionAdvice"
        self.__exception_advice_cls: type = exception_advice_cls

    def __call__(self):
        self.__exception_advice_cls()

    @classmethod
    def throws_exception(cls, ex_type: type):
        if not isinstance(ex_type, type):
            raise TypeError("ThrowsException: argument 'ex' must be a type")

        def decorator_throws_exception_func(func) -> callable:
            func.__decorator__ = "ThrowsException"
            func.__decorator_params__ = ex_type
            return func

        return decorator_throws_exception_func


class RequestInterceptor:
    def __init__(self, interceptor_paths: set[str]) -> None:
        if not isinstance(interceptor_paths, set):
            raise TypeError('RequestInterceptor: interceptor_paths must be a set')
        for interceptor_path in interceptor_paths:
            if not isinstance(interceptor_path, str):
                raise TypeError('RequestInterceptor: interceptor_paths must be a string set')
        self.__interceptor_paths: set[str] = interceptor_paths

    def __call__(self, interceptor_cls: type) -> type:
        if not isinstance(interceptor_cls, type):
            raise TypeError('RequestInterceptor can only be applied to classes')
        interceptor_cls.__decorator__ = "RequestInterceptor"
        interceptor_cls.__decorator_params__ = self.__interceptor_paths
        return interceptor_cls

    @classmethod
    def interceptor_before(cls) -> callable:
        def decorator_request_func(func) -> callable:

            def interceptor_before_method(it_self: object, request: BaseHTTPRequestHandler) -> tuple[int, bool]:
                if (not isinstance(request, BaseHTTPRequestHandler) and
                        not isinstance(it_self, BaseHTTPRequestHandler)):
                    raise ArgumentTypeError('InterceptorBefore: argument must be BaseHTTPRequestHandler Type')
                request.headers["X-Intercepted"] = "True"
                response_code, response_status = func(it_self, request)
                if not isinstance(response_code, int) or not isinstance(response_status, bool):
                    err: str = 'InterceptorBefore: return value must be (response_code: int, response_status: bool)'
                    raise ValueError(err)
                return response_code, response_status

            interceptor_before_method.__decorator__ = "InterceptorBefore"
            return interceptor_before_method

        return decorator_request_func

    @classmethod
    def interceptor_after(cls) -> callable:
        def decorator_request_func(func) -> callable:
            def interceptor_after_method(it_self: object, request: BaseHTTPRequestHandler) -> None:
                if not isinstance(request, BaseHTTPRequestHandler) and not isinstance(it_self,
                                                                                      BaseHTTPRequestHandler):
                    raise ArgumentTypeError('InterceptorAfter: argument should be BaseHTTPRequestHandler Type')
                func(it_self, request)
                return None

            interceptor_after_method.__decorator__ = "InterceptorAfter"
            return interceptor_after_method

        return decorator_request_func


resource = ContainerAnnotation.resource
get = ContainerAnnotation.get
post = ContainerAnnotation.post
put = ContainerAnnotation.put
delete = ContainerAnnotation.delete
throws_exception = ExceptionAdvice.throws_exception
before = RequestInterceptor.interceptor_before
after = RequestInterceptor.interceptor_after



# @Coding: UTF-8
# @Time: 2024/9/20 12:20
# @Author: xieyang_ls
# @Filename: __init__.py.py

from pyutils_spirit.tcp.websocket import WebSocketServer, EndPoint, onopen, onmessage, onerror, onclose, Session

__all__ = ['WebSocketServer',
           'EndPoint',
           'onopen',
           'onmessage',
           'onerror',
           'onclose',
           'Session']



import json

from datetime import datetime

from pyutils_spirit.util.json_util import deep_dumps


class Chat:

    def __init__(self):
        self.id: str | None = None
        self.avatar: str | None = None
        self.chat_name: str | None = None
        self.message: Message | None = None
        self.create_time: datetime | None = datetime.now()

    def to_dict(self) -> dict:
        return {
            "id": self.id,
            "avatar": self.avatar,
            "chat_name": self.chat_name,
            "message": None if self.message is None else self.message.to_dict(),
            "create_time": self.create_time
        }

    def __str__(self) -> str:
        return str(self.to_dict())


class Message:

    def __init__(self):
        self.id: str | None = None
        self.sender: str | None = None
        self.receiver: str | None = None
        self.group: str | None = None
        self.content: str | bytes | None = None
        self.msg_type: int | None = None
        self.location: Location | None = None
        self.reference_msg: str | None = None
        self.create_time: datetime | None = datetime.now()

    def to_dict(self) -> dict:
        return {
            "id": self.id,
            "sender": self.sender,
            "receiver": self.receiver,
            "group": self.group,
            "content": self.content,
            "msg_type": self.msg_type,
            "location": None if self.location is None else self.location.to_dict(),
            "reference_msg": self.reference_msg,
            "create_time": self.create_time
        }

    def __str__(self) -> str:
        return str(self.to_dict())


class Location:

    def __init__(self):
        self.id: str | None = None
        self.address: str | None = None
        self.latitude: float | None = None
        self.longitude: float | None = None
        self.create_time: datetime = datetime.now()

    def to_dict(self) -> dict:
        return {
            "id": self.id,
            "address": self.address,
            "latitude": self.latitude,
            "longitude": self.longitude,
            "create_time": self.create_time
        }

    def __str__(self) -> str:
        return str(self.to_dict())



# @Coding: UTF-8
# @Time: 2024/9/22 17:13
# @Author: xieyang_ls
# @Filename: __init__.py.py

from pyutils_spirit.spirit_container.annotation import (Component,
                                                        Mapper,
                                                        Service,
                                                        Controller,
                                                        resource,
                                                        get, post, put, delete,
                                                        ExceptionAdvice,
                                                        throws_exception,
                                                        RequestInterceptor,
                                                        before,
                                                        after)

from pyutils_spirit.spirit_container.multipart_file import MultipartFile

from pyutils_spirit.spirit_container.request_result import Result

from pyutils_spirit.spirit_container.spirit_application import SpiritApplication

from pyutils_spirit.spirit_container.spirit_application_container import SpiritApplicationContainer

__all__ = ["Component",
           "Mapper",
           "Service",
           "Controller",
           "resource",
           "get",
           "post",
           "put",
           "delete",
           "ExceptionAdvice",
           "throws_exception",
           "RequestInterceptor",
           "before",
           "after",
           "MultipartFile",
           "Result",
           "SpiritApplication",
           "SpiritApplicationContainer"]



from typing import Any

from pyutils_spirit.util.assemble import Assemble, HashAssemble

from pyutils_spirit.exception.exception import ArgumentException

from pyutils_spirit.util.cities import get_all_cities, get_cities, get_provinces


class SpiritApplicationContainer:
    __instance: Any = None

    __application_container: Assemble[str, Any] = None

    def __init__(self):
        if self.__application_container is None:
            self.__application_container = HashAssemble()

    def __new__(cls) -> object:
        if cls.__instance is None:
            cls.__instance = object.__new__(cls)
        return cls.__instance

    def set_resource(self, signature: str, resource: Any):
        if signature in self.__application_container:
            raise ArgumentException(f"spirit_application_container: '{signature}' "
                                    f"Different resources cannot have the same signature")
        self.__application_container[signature] = resource

    def get_resource(self, signature: str) -> Any:
        return self.__application_container[signature]

    def remove_resource(self, signature: str) -> Any:
        return self.__application_container.remove(key=signature)

    def get_capacity(self):
        return len(self.__application_container)

    @classmethod
    def wired_container(cls, signature: str, resource_cls: type) -> None:
        if cls.__instance is None:
            cls.__instance = SpiritApplicationContainer()

        def wired_resource(other_cls) -> object:
            instance = cls.__instance.get_resource(signature=signature)
            if instance is None:
                instance = object.__new__(other_cls)
                cls.__instance.set_resource(signature=signature, resource=instance)
            return instance

        resource_cls.__new__ = wired_resource
        resource_cls()

    def __len__(self):
        return len(self.__application_container)

    def __iter__(self):
        return iter(self.__application_container)

    def __next__(self):
        return next(self.__application_container)


class CityComponent:
    def __init__(self):
        self.get_cities: callable = get_cities
        self.get_province: callable = get_provinces
        self.get_all_cities: callable = get_all_cities


SpiritApplicationContainer.wired_container(signature="city_component", resource_cls=CityComponent)
